// go run v2.go -- $PWD
package main

import (
	"bufio"
	"flag"
	"fmt"
	"os"
	"path/filepath"
)

var globalLineCount int

func lineCount(path string) (int, error) {
	f, err := os.Open(path)
	if err != nil {
		return 0, err
	}
	defer f.Close()
	r := bufio.NewReader(f)
	var lc int
	for err == nil {
		_, err = r.ReadString('\n')
		if err == nil {
			lc++
		}
	}
	return lc, nil
}

// START visit OMIT
func visit(path string, fi os.FileInfo, err error) error {
	if err != nil {
		return err
	}
	if !fi.Mode().IsRegular() {
		return nil
	}
	lc, err := lineCount(path)
	if err != nil {
		return err
	}
	globalLineCount += lc
	fmt.Printf("%d\t%s\n", lc, path)
	return nil
}

// END visit OMIT

func main() {
	flag.Parse()
	// START main OMIT
	root := flag.Arg(0)
	err := filepath.Walk(root, visit)
	// END main OMIT
	if err != nil {
		fmt.Errorf("%s", err)
		os.Exit(1)
	}
	fmt.Printf("%d\ttotal\n", globalLineCount)
}
